// SPDX-FileCopyrightText: 2024 Yury Gubich <blue@macaw.me>
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <string>
#include <map>

#include "gloox/gloox.h"

#include "yaml-cpp/yaml.h"

#include "shared/definitions.h"
#include "shared/result.h"
#include "logger.h"

class Config {
public:
    struct Module {
        bool enabled;
        std::string alias;
        Shared::Permissions permissions;
    };

public:
    Config(const std::string& path);

    bool isValid() const;

    std::string getBareJID() const;
    std::string getFullJID() const;
    std::string getPassword() const;
    std::string getResource() const;
    std::map<std::string, std::string> getActors() const;
    Logger::Level getLogLevel() const;
    gloox::TLSPolicy getTLSPolicy() const;
    Module getModuleConfig(const std::string& name) const;
    Shared::Responses getResponses() const;

private:
    YAML::Node root;
};
