// SPDX-FileCopyrightText: 2024 Yury Gubich <blue@macaw.me>
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <string>
#include <map>
#include <vector>
#include <memory>
#include <random>

#include "shared/result.h"
#include "actor.h"
#include "logger.h"

namespace Module {
    class Module;
};

class Connection;

class Router {
public:
    Router(const Logger& logger);

    void registerModule(const std::string& key, const std::shared_ptr<Module::Module>& module);
    void registerActor(const std::string& key, const std::string& group);
    void routeMessage(const std::string& sender, const std::string& body);

    std::map<std::string, std::string> getActors() const;

    void setConnection(const std::shared_ptr<Connection>& connection);
    void setResponses(Shared::Result result, const Shared::Strings& options);
    std::string getGroup(const std::string& key) const;

private:
    void onMessageResult(Shared::Result result, const std::string& sender);

private:
    typedef std::map<std::string, std::weak_ptr<Module::Module>> Modules;
    typedef std::map<std::string, std::shared_ptr<Actor>> Actors;
    typedef std::vector<std::string> List;
    typedef std::map<Shared::Result, List> Responses;

    std::weak_ptr<Connection> connection;
    const Logger& logger;
    Modules modules;
    Actors actors;
    std::string defaultGroup;
    Responses responses;
    std::mt19937 generator;
};
